const { VITE_APP_ADMIN_API, VITE_APP_MC_API } = import.meta.env;

import Axios, {
  AxiosInstance,
  AxiosRequestConfig,
  CustomParamsSerializer,
} from "axios";
import { showFailToast } from "vant";
import {
  getToken,
  removeToken,
  getMechanismToken,
  removeMechanismToken,
  getSurveyToken,
  removeSurveyToken,
} from "@/utils/auth";
import {
  PureHttpError,
  RequestMethods,
  PureHttpResponse,
  PureHttpRequestConfig,
} from "./types.d";
import authWhite from "./authWhite";
import { stringify } from "qs";
import NProgress from "../progress";
import router from "@/router";
// 相关配置请参考：www.axios-js.com/zh-cn/docs/#axios-request-config-1
const defaultConfig: AxiosRequestConfig = {
  baseURL: VITE_APP_MC_API,
  // 请求超时时间
  timeout: 60000,
  headers: {
    Accept: "application/json, text/plain, */*",
    "Content-Type": "application/json",
    "X-Requested-With": "XMLHttpRequest",
  },
  // 数组格式参数序列化（https://github.com/axios/axios/issues/5142）
  paramsSerializer: {
    serialize: stringify as unknown as CustomParamsSerializer,
  },
};

class PureHttp {
  constructor() {
    this.httpInterceptorsRequest();
    this.httpInterceptorsResponse();
  }

  /** token过期后，暂存待执行的请求 */
  private static requests = [];

  /** 防止重复刷新token */
  private static isRefreshing = false;

  /** 初始化配置对象 */
  private static initConfig: PureHttpRequestConfig = {};

  /** 保存当前Axios实例对象 */
  private static axiosInstance: AxiosInstance = Axios.create(defaultConfig);

  /** 重连原始请求 */
  private static retryOriginalRequest(config: PureHttpRequestConfig) {
    return new Promise((resolve) => {
      PureHttp.requests.push((token: string) => {
        // config.headers[tokenKey] = formatToken(token);
        resolve(config);
      });
    });
  }

  /** 请求拦截 */
  private httpInterceptorsRequest(): void {
    const domain = {
      "/algor/": VITE_APP_ADMIN_API + "/",
      "/ocrService/": VITE_APP_ADMIN_API + "/ocrService/",
      "/graphService/": VITE_APP_ADMIN_API + "/graphService/",
    };
    PureHttp.axiosInstance.interceptors.request.use(
      async (config: PureHttpRequestConfig): Promise<any> => {
        config.timeout = defaultConfig.timeout
        // 开启进度条动画
        NProgress.start();
        const _data = config.params || config.data;
        const findReplaceDomainKey = Object.keys(domain).find((k) =>
          new RegExp(`^${k}`).test(config.url)
        );
        console.log("request config ", config);
        if (config.url.includes("/ocrService/setting/ocr/status")) {
          config.timeout = 2000
        }
        if (
          authWhite.findIndex((v) => new RegExp(`^${v}`).test(config.url)) > -1
        ) {
          // config.url = VITE_APP_AUTH_ADMIN_API + config.url;
        } else if (findReplaceDomainKey) {
          config.url = config.url.replace(
            new RegExp(`^${findReplaceDomainKey}`),
            domain[findReplaceDomainKey]
          );
        }
        const tk = getToken();
        console.log('请求tk', tk)
        if (tk && tk.token) {
          config.headers["token"] = tk.token;
        }
        const mTk = getMechanismToken();
        if (mTk) {
          config.headers["mechanismToken"] = mTk;
        }

        if (localStorage.getItem("mechanismId")) {
          config.headers!["mechanismId"] = localStorage.getItem("mechanismId")!;
        }

        const sTk = getSurveyToken();
        if (sTk) {
          config.headers!["surveyToken"] = sTk;
        }
        config.headers["platform"] = "Mechanism";
        if (typeof config.beforeRequestCallback === "function") {
          // 优先判断post/get等方法是否传入回调，否则执行初始化设置等回调
          config.beforeRequestCallback(config);
          return config;
        }
        return config;
      },
      (error) => {
        return Promise.reject(error);
      }
    );
  }

  /** 响应拦截 */
  private httpInterceptorsResponse(): void {
    const instance = PureHttp.axiosInstance;
    instance.interceptors.response.use(
      (response: PureHttpResponse) => {
        const $config = response.config;
        console.log(" $config", $config);

        const { data } = response as any;
        const { code, message } = data;
        // 关闭进度条动画
        NProgress.done();
        if (code === undefined) {
          return data;
        } else {
          switch (Number(code)) {
            case 200:
            case 202:
              return data;
            case 80002:
              return data;
            case 401:
            case 403:
              showFailToast(message);
              removeToken();
              removeMechanismToken();
              removeSurveyToken();
              const entry = sessionStorage.getItem(`entry`);
              if (entry == "archivesIndex") {
                router.replace({ name: "mechanismMobileLogin" });
              } else {
                router.replace({ name: "login" });
              }
              return;
            case 9004:
            case 9006:
              showFailToast(message);
              router.replace({
                name: "fillInfo",
                query: {
                  code,
                },
              });
              return;
            case 9100:
              showFailToast(message);
              return data;
            default:
              if (!["/mallService/api/address/default"].includes($config.url)) {
                showFailToast(message);
              }
              // ElMessage.error(message);
              return Promise.reject(data);
          }
        }
      },
      (error: PureHttpError) => {
        const $error = error;
        $error.isCancelRequest = Axios.isCancel($error);
        // 关闭进度条动画
        NProgress.done();
        // 所有的响应异常 区分来源为取消请求/非取消请求
        return Promise.reject($error);
      }
    );
  }

  /** 通用请求工具函数 */
  public request<T>(
    method: RequestMethods,
    url: string,
    param?: AxiosRequestConfig,
    axiosConfig?: PureHttpRequestConfig
  ): Promise<T> {
    const config = {
      method,
      url,
      ...param,
      ...axiosConfig,
    } as PureHttpRequestConfig;

    // 单独处理自定义请求/响应回调
    return new Promise((resolve, reject) => {
      PureHttp.axiosInstance
        .request(config)
        .then((response: undefined) => {
          resolve(response);
        })
        .catch((error) => {
          reject(error);
        });
    });
  }

  /** 单独抽离的post工具函数 */
  public post<T, P>(
    url: string,
    params?: AxiosRequestConfig<T>,
    config?: PureHttpRequestConfig
  ): Promise<P> {
    return this.request<P>("post", url, { data: params }, config);
  }
  public put<T, P>(
    url: string,
    params?: AxiosRequestConfig<T>,
    config?: PureHttpRequestConfig
  ): Promise<P> {
    return this.request<P>("put", url, { data: params }, config);
  }
  public delete<T, P>(
    url: string,
    params?: AxiosRequestConfig<T>,
    config?: PureHttpRequestConfig
  ): Promise<P> {
    return this.request<P>("delete", url, { data: params }, config);
  }

  /** 单独抽离的get工具函数 */
  public get<T, P>(
    url: string,
    params?: AxiosRequestConfig<T>,
    config?: PureHttpRequestConfig
  ): Promise<P> {
    return this.request<P>("get", url, params, config);
  }
}

export const request = new PureHttp();
